@@ -221,6 +221,8 @@ static gboolean     gtk_file_folder_gnome_vfs_list_children (GtkFileFolder
 							     GSList            **children,
 							     GError            **error);
 
+static gboolean     gtk_file_folder_gnome_vfs_is_finished_loading (GtkFileFolder *folder);
+
 static GtkFileInfo *           info_from_vfs_info (const gchar      *uri,
 						   GnomeVFSFileInfo *vfs_info,
 						   GtkFileInfoType   types);
@@ -375,14 +377,14 @@ gtk_file_system_gnome_vfs_init (GtkFileSystemGnomeVFS *system_vfs)
 #endif
 
   system_vfs->volume_monitor = gnome_vfs_volume_monitor_ref (gnome_vfs_get_volume_monitor ());
-  g_signal_connect (system_vfs->volume_monitor, "volume-mounted",
-		    G_CALLBACK (volume_mount_unmount_cb), system_vfs);
-  g_signal_connect (system_vfs->volume_monitor, "volume-unmounted",
-		    G_CALLBACK (volume_mount_unmount_cb), system_vfs);
-  g_signal_connect (system_vfs->volume_monitor, "drive-connected",
-		    G_CALLBACK (drive_connect_disconnect_cb), system_vfs);
-  g_signal_connect (system_vfs->volume_monitor, "drive-disconnected",
-		    G_CALLBACK (drive_connect_disconnect_cb), system_vfs);
+  g_signal_connect_object (system_vfs->volume_monitor, "volume-mounted",
+			   G_CALLBACK (volume_mount_unmount_cb), system_vfs, 0);
+  g_signal_connect_object (system_vfs->volume_monitor, "volume-unmounted",
+			   G_CALLBACK (volume_mount_unmount_cb), system_vfs, 0);
+  g_signal_connect_object (system_vfs->volume_monitor, "drive-connected",
+			   G_CALLBACK (drive_connect_disconnect_cb), system_vfs, 0);
+  g_signal_connect_object (system_vfs->volume_monitor, "drive-disconnected",
+			   G_CALLBACK (drive_connect_disconnect_cb), system_vfs, 0);
 
 }
 
@@ -938,7 +940,7 @@ gtk_file_system_gnome_vfs_volume_render_icon (GtkFileSystem        *file_system,
     {
       uri = gnome_vfs_volume_get_activation_uri (GNOME_VFS_VOLUME (volume));
       if (strcmp (uri, "file:///") == 0)
-	icon_name = g_strdup ("gnome-fs-blockdev");
+	icon_name = g_strdup ("gnome-fs-harddisk");
       else if (strcmp (uri, system_vfs->desktop_uri) == 0)
 	icon_name = g_strdup ("gnome-fs-desktop");
       else if (strcmp (uri, system_vfs->home_uri) == 0)
@@ -1760,6 +1762,7 @@ gtk_file_folder_gnome_vfs_iface_init (GtkFileFolderIface *iface)
 {
   iface->get_info = gtk_file_folder_gnome_vfs_get_info;
   iface->list_children = gtk_file_folder_gnome_vfs_list_children;
+  iface->is_finished_loading = gtk_file_folder_gnome_vfs_is_finished_loading;
 }
 
 static void
@@ -1907,6 +1910,14 @@ gtk_file_folder_gnome_vfs_list_children (GtkFileFolder  *folder,
   return TRUE;
 }
 
+static gboolean
+gtk_file_folder_gnome_vfs_is_finished_loading (GtkFileFolder *folder)
+{
+  GtkFileFolderGnomeVFS *folder_vfs = GTK_FILE_FOLDER_GNOME_VFS (folder);
+
+  return (folder_vfs->async_handle == NULL);
+}
+
 
 static GnomeVFSFileInfoOptions
 get_options (GtkFileInfoType types)
@@ -2114,7 +2125,11 @@ directory_load_callback (GnomeVFSAsyncHandle *handle,
     }
 
   if (result != GNOME_VFS_OK)
-    folder_vfs->async_handle = NULL;
+    {
+      folder_vfs->async_handle = NULL;
+
+      g_signal_emit_by_name (folder_vfs, "finished-loading");
+    }
 }
 
 static void
@@ -2331,6 +2346,15 @@ fs_module_init (GTypeModule    *module)
 				 GTK_TYPE_FILE_FOLDER,
 				 &file_folder_info);
   }
+
+  /* We ref the class so that the module won't get unloaded.  We need this
+   * because gnome_icon_lookup() will cause GnomeIconTheme to get registered,
+   * and we cannot unload that properly.  I'd prefer to do this via
+   * g_module_make_resident(), but here we don't have access to the GModule.
+   * See bug #139254 for reference. [federico]
+   */
+
+  g_type_class_ref (type_gtk_file_system_gnome_vfs);
 }
 
 void 
